/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibañez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package org.gvsig.publish.geoserver.conf;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.geoserver.model.Geoserver;

/**
 * This class represents the Geoserver Configuration.
 * It has the responsibility to generate the Geoserver XML configuration files (servers.xml, catalog.xml, info.xml) 
 * If the configuration exists it will be removed
 * @author jvhigon (josevicente.higon@iver.es)
 */
public class GSConfiguration  {
	private Geoserver model = null;
	//Attributes from associations
	private GSServer serverConf = null; 
	private Map featureTypes = null;
	private GSCatalog catalog;
		

	/**
	 * Default constructor
	 */
	public GSConfiguration (Geoserver geoserver){
		this.model = geoserver;
		this.serverConf = new GSServer(geoserver);
		this.featureTypes = new HashMap();
		this.catalog = new GSCatalog();
	}

	/**
	 * 
	 * @return object that represents Geoserver's services
	 */
	public GSServer getGlobals(){
		return this.serverConf;
	}

	/**
	 * Reloads Geoserver in order to apply changes in the configuration 
	 * @param serverurl For example http://localhost:8080/geoserver
	 * @throws IOException 
	 */
	public void reload(URL serverurl, String user, String password) throws IOException {

		String aux = serverurl.toString() + "/admin/loginSubmit.do?username="+user+"&password="+password+"&submit=Submit";
		URL url = new URL(aux);
		URLConnection conn = url.openConnection();
		InputStream inStream = conn.getInputStream();
		String responseString=new String();
		BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
		while(in.ready()){   
			responseString+= in.readLine();  
		}						
		// reload
		String cookie=conn.getHeaderField("Set-Cookie");

		cookie = cookie.substring(0, cookie.indexOf(";"));
		String cookieName = cookie.substring(0, cookie.indexOf("="));
		String cookieValue = cookie.substring(cookie.indexOf("=") + 1, cookie.length());
		String cookieString=cookieName+"="+cookieValue;
		URL url2=new URL(serverurl.toString()+"/admin/loadFromXML.do");
		URLConnection conn2 = url2.openConnection();
		conn2.setRequestProperty("Cookie",cookieString);                 // set the Cookie for request
		conn2.connect();
		inStream = conn2.getInputStream();
		in = new BufferedReader(new InputStreamReader(inStream));
		responseString=new String();
		while(in.ready()){  
			responseString+= in.readLine();  
		}						
		//logout
		URL url3=new URL(serverurl.toString()+"/admin/logout.do");
		URLConnection conn3 = url3.openConnection();
		conn3.setRequestProperty("Cookie",cookieString);
		conn3.connect();
		inStream = conn3.getInputStream();
		in = new BufferedReader(new InputStreamReader(inStream));
		responseString=new String();
		while(in.ready()){   
			responseString+= in.readLine(); 
		}
	}


	/**
	 * Adds a new feature for publishing.
	 * Adds a datastore, namespace and style into the catalog 
	 * 
	 * @param feature
	 * @return false if the datasoruce already exists
	 */
	public boolean addFeatureType(GSFeature feature){
		if (featureTypes.containsKey(feature.getId())==true){
			PublishLogger.getLog().error("ERROR " + getClass().getName() + ": This feature already exists");
		}
		featureTypes.put(feature.getId(), feature);
		boolean alreadyExists = catalog.addDatastore(feature.getDatastore());		
		catalog.addNamespace(feature.getNamespace());
		catalog.addStyle(feature.getStyle());
		if (alreadyExists){
			return false;
		}else{
			return true;
		}
	}

	/**
	 * Writes the Geoserver config files
	 * @throws IOException 
	 * 
	 */
	public void toXML() throws IOException{        
		File dir = model.getConfigDirectory();
		//generates catalog.xml
		File catalog_xml = new File(dir.getAbsolutePath() + File.separator +"catalog.xml");
		this.catalog.toXML(catalog_xml);
		//generates services.xml
		File services_xml = new File(dir.getAbsolutePath() + File.separator +"services.xml");
		this.serverConf.toXML(services_xml);        
		//clean directories        
		File feature_dir = new File(dir.getAbsoluteFile()+ File.separator + "featureTypes");
		File styles_dir = new File(dir.getAbsolutePath() + File.separator + "styles");
		if (feature_dir.exists()){        	
			removeDirectory(feature_dir);        	
		}
		feature_dir.mkdirs();
		if (styles_dir.exists()){
			removeDirectory(styles_dir);	
		}
		styles_dir.mkdir();
		//generates info.xml (and write sld into directory styles)
		Iterator i = featureTypes.values().iterator();
		while (i.hasNext()){
			GSFeature feature = (GSFeature)i.next();        	
			feature.toXML(feature_dir);           	        	
		}             
		//creates directory coverages & plugIns
		File coverages_dir = new File(dir.getAbsoluteFile()+ File.separator + "coverages");
		if (!coverages_dir.exists()) coverages_dir.mkdirs();
		File plugins_dir = new File(dir.getAbsoluteFile()+ File.separator + "plugIns");
		if (!plugins_dir.exists()) plugins_dir.mkdirs();
	}
	/**
	 * Delete all directory contents
	 * @param dir
	 */
	private void removeDirectory(File dir){
		File[] files = dir.listFiles();
		if (files != null){
			for (int i = 0; i < files.length; i++){
				removeDirectory(files[i]);
			}    	
		}    
		dir.delete();
	}

}
