/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibañez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package org.gvsig.publish.geoserver.conf;

import java.net.URL;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * This class represents a datastore in the Geoserver Configuration.
 * <p>
 * A datastore configuration element serves as a common data source connection
 * parameters repository for all featuretypes it holds.
 * 
 * @author jvhigon (josevicente.higon@iver.es)
 */
public class GSDatastore  {
	//Association
	private GSNamespace namespace = null;
	//Attributes
	private String id;
	private boolean isEnabled;
	private String charset;
	private URL url= null;
	private String abstrac;
	private String user;
	private String password;
	private String host;
	private String schema;
	private String database;
	private String dbType;
	private boolean looseBbox;
	private boolean estimatedExtent;
	private int port;
	
    /**
     * Default constructor.
     * <p>
     * Initializes some variables with default values
     */
    public GSDatastore (){
    	isEnabled = true;
    }
    /**
     * 
     * @return id unique value to indentify the datastore
     */
    public String getId(){
    	return this.id;
    }

    /**
     * Sets the data-store identifier. It must be unique.
     * @param id 
     */
    public void setId(String id) {
    	this.id = id;
    }
	/**
	 * 
	 * @return true if the datastore is enabled
	 */
    public boolean getIsEnabled(){
    	return this.isEnabled;
    }
    /**
     * Active/desactive the datastore
     * 
     * @param isEnabled  
     */
    public void setIsEnabled(boolean isEnabled) {
    	this.isEnabled = isEnabled;
    }
	
    /**
     * 
     * @param ns 
     */
    public void setNamespace(GSNamespace ns) {
    	this.namespace = ns;
    }
	
    /**
     * Sets the URL which points to the datastore
     * <p>
     * Example: file:data/sf/roads.shp 
     * 
     * @param url 
     */
    public void setURL(URL url) {
    	this.url = url;
    }
	
    /**
     * Sets the datastores charset (UTF-8, ISO-8859-1,...)
     * 
     * @param charset (UTF-8, ISO-8859-1, ...)
     */
    public void setCharset(String charset) {
    	this.charset = charset;
    }
    
	/**
	 * 
	 * @return datastore element to insert into a XML
	 */
	public Element getXMLElement(Document dom) {
		//set root element
		Element rootEle = dom.createElement("datastore");
		//set attributes
		rootEle.setAttribute("id", this.getId());
		rootEle.setAttribute("enabled", new Boolean(this.getIsEnabled()).toString());
		rootEle.setAttribute("namespace", this.namespace.getPrefix());

		//set abstract 		
		Element paramEle = dom.createElement("abstract");
		Text txt = dom.createTextNode(this.getAbstract());
		paramEle.appendChild(txt);			
		rootEle.appendChild(paramEle);
		
		//set connectionParams
		Element conEle = dom.createElement("connectionParams");
		rootEle.appendChild(conEle);
		//set charset
		if (this.charset != null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "charset");
			paramEle.setAttribute("value", this.charset);
			conEle.appendChild(paramEle);
		}		
		//set url
		if (url !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "url");
			paramEle.setAttribute("value", this.url.toString());
			conEle.appendChild(paramEle);
		}
		
		//set namespace
		paramEle = dom.createElement("parameter");
		paramEle.setAttribute("name", "namespace");
		paramEle.setAttribute("value", this.namespace.getPrefix());
		conEle.appendChild(paramEle);
		
		//set user
		if (user !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "user");
			paramEle.setAttribute("value", getUser());
			conEle.appendChild(paramEle);
		}
		//set password
		if (password !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "password");
			paramEle.setAttribute("value", getPassword());
			conEle.appendChild(paramEle);
		}
		//set host
		if (host !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "host");
			paramEle.setAttribute("value", getHost());
			conEle.appendChild(paramEle);
		}
		//set schema
		if (schema !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "schema");
			paramEle.setAttribute("value", getSchema());
			conEle.appendChild(paramEle);
		}
		//set database
		if (database !=null){
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "database");
			paramEle.setAttribute("value", getDatabase());
			conEle.appendChild(paramEle);
		}
		//set database ...
		if (dbType !=null){
			//dbType
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "dbtype");
			paramEle.setAttribute("value", getDbType());
			conEle.appendChild(paramEle);
			//set looseBbox
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "looseBbox");
			paramEle.setAttribute("value", new Boolean(isLooseBbox()).toString());
			conEle.appendChild(paramEle);
			//set estimatedExtent
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "estimatedExtent");
			paramEle.setAttribute("value", new Boolean(isEstimatedExtent()).toString());
			conEle.appendChild(paramEle);
			//set port			
			paramEle = dom.createElement("parameter");
			paramEle.setAttribute("name", "port");
			paramEle.setAttribute("value", new Integer(getPort()).toString());
			conEle.appendChild(paramEle);
		}

		return rootEle;
	}
	/**
	 * @return the url
	 */
	public URL getUrl() {
		return url;
	}
	/**
	 * @param url the url to set
	 */
	public void setUrl(URL url) {
		this.url = url;
	}
	/**
	 * @return the abstrac
	 */
	public String getAbstract() {
		return abstrac;
	}
	/**
	 * @param abstrac the abstrac to set
	 */
	public void setAbstract(String abstrac) {
		this.abstrac = abstrac;
	}
	/**
	 * @return the user
	 */
	public String getUser() {
		return user;
	}
	/**
	 * @param user the user to set
	 */
	public void setUser(String user) {
		this.user = user;
	}
	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}
	/**
	 * @param password the password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	/**
	 * @return the host
	 */
	public String getHost() {
		return host;
	}
	/**
	 * @param host the host to set
	 */
	public void setHost(String host) {
		this.host = host;
	}
	/**
	 * @return the schema
	 */
	public String getSchema() {
		return schema;
	}
	/**
	 * @param schema the schema to set
	 */
	public void setSchema(String schema) {
		this.schema = schema;
	}
	/**
	 * @return the database
	 */
	public String getDatabase() {
		return database;
	}
	/**
	 * @param database the database to set
	 */
	public void setDatabase(String database) {
		this.database = database;
	}
	/**
	 * @return the dbType
	 */
	public String getDbType() {
		return dbType;
	}
	/**
	 * @param dbType the dbType to set
	 */
	public void setDbType(String dbType) {
		this.dbType = dbType;
	}
	/**
	 * @return the namespace
	 */
	public GSNamespace getNamespace() {
		return namespace;
	}
	/**
	 * @return the charset
	 */
	public String getCharset() {
		return charset;
	}
	/**
	 * @param isEnabled the isEnabled to set
	 */
	public void setEnabled(boolean isEnabled) {
		this.isEnabled = isEnabled;
	}
	/**
	 * @return the looseBbox
	 */
	public boolean isLooseBbox() {
		return looseBbox;
	}
	/**
	 * @param looseBbox the looseBbox to set
	 */
	public void setLooseBbox(boolean looseBbox) {
		this.looseBbox = looseBbox;
	}
	/**
	 * @return the estimatedExtent
	 */
	public boolean isEstimatedExtent() {
		return estimatedExtent;
	}
	/**
	 * @param estimatedExtent the estimatedExtent to set
	 */
	public void setEstimatedExtent(boolean estimatedExtent) {
		this.estimatedExtent = estimatedExtent;
	}
	/**
	 * @return the port
	 */
	public int getPort() {
		return port;
	}
	/**
	 * @param port the port to set
	 */
	public void setPort(int port) {
		this.port = port;
	}
		
}
