/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibañez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.geoserver.model;


import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.geoserver.conf.GSDatastore;
import org.gvsig.publish.geoserver.conf.GSFeature;
import org.gvsig.publish.geoserver.conf.GSNamespace;
import org.gvsig.publish.geoserver.conf.GSStyle;
import org.gvsig.publish.infoproject.IDataSourceInfo;
import org.gvsig.publish.infoproject.ILayerInfo;
import org.gvsig.publish.infoproject.datasources.IDataBaseInfo;
import org.gvsig.publish.infoproject.datasources.IFileInfo;
import org.gvsig.publish.ogcmetadata.IOGCCommonMetadata;
import org.gvsig.publish.ogcmetadata.OGCCommonMetadata;
import org.gvsig.publish.serversmodel.RemoteResource;

import com.iver.utiles.XMLEntity;
/**
 * This class represents a remote resource (a feature) which can be offered by Geoserver
 * This remote resource will be able to be accessed through WMS or WFS. 
 * It has the responsibility of persisting the configuration.
 * 
 * @author jvhigon
 *
 */
public class GeoserverFeature extends RemoteResource implements IOGCCommonMetadata {
	//Attributes
	private OGCCommonMetadata ogcMetadata= null;
	private String namespacePrefix =null;
	public static final String REGISTER_TAG="geoserver_feature";
	private boolean cacheEnabled=true;
	private int cacheMaxage=3600;
	private int numDecimals=0;

	/**
	 * constructor
	 * @param service
	 */
	public GeoserverFeature(GeoserverWFSService service) {
		super(service);		
		ogcMetadata = new OGCCommonMetadata();
	}

	/** 
	 * Overrides the method in order to initialize some variables like namespacePrefix.
	 * Also overrides the method in order to avoid the creation of nested resources 
	 * @see org.gvsig.publish.serversmodel.RemoteResource#setLayerInfo(org.gvsig.publish.infoproject.ILayerInfo)
	 */

	public void setLayerInfo(ILayerInfo ilayer) {
		super.setLayerInfo(ilayer);
		setNamespacePrefix(getLayerInfo().getDataSource().getNamespacePrefix());
		setName(getFeatureTypeName());
		setTitle(getLayerInfo().getTitle());
		setAbstract("generated by gvSIG");
	}
	/**
	 * 
	 * @return feature name in Geoserver. It's the filename without extension for files or tablename for databases
	 * 
	 */
	private String getFeatureTypeName() {
		String aux="unknowfeature";
		IDataSourceInfo ds = getLayerInfo().getDataSource();
		if (ds instanceof IFileInfo){
			IFileInfo ifile = (IFileInfo)ds;
			aux = ifile.getFilenameWithoutExtension();
		}else{
			if (ds instanceof IDataBaseInfo){
				IDataBaseInfo idb = (IDataBaseInfo) ds;
				aux = idb.getTableName();
			}
		}
		return aux;
	}
	/*
	 * (non-Javadoc)
	 * @see com.iver.utiles.IPersistance#getClassName()
	 */
	public String getClassName() {		
		return "GeoserverFeature";
	}

	/**
	 * 
	 * @see org.gvsig.publish.serversmodel.RemoteResource#publish()
	 */
	public void publish() throws PublishException{		
		//adds this feature to the server configuration		
		addFeatureToConfig(); 
		//adds its childs to the server configuration
		super.publish();
	}
	/**
	 * Adds this remote resource into the server configuration
	 * @throws PublishException  
	 */
	private void addFeatureToConfig() throws PublishException{
		GSFeature config = new GSFeature(this);
		//create namespace
		GSNamespace ns = create_namespace();
		//create datastore
		GSDatastore ds = create_datastore(config, ns);
		//create style 
		GSStyle st = create_style();
		//create feature
		config.setDatastore(ds);
		config.setNamespace(ns);
		config.setStyle(st);

		//adds feature
		Geoserver server= (Geoserver) getService().getServer();

		boolean isadded = server.getConfiguration().addFeatureType(config);
		if (isadded == false && getLayerInfo().getDataSource().getType()==IDataSourceInfo.SHAPE_TYPE){
			PublishLogger.getLog().error("ERROR GeoserverFeature: cannot add a feature because you can't have two features with the same shapefile in the same namespace");
			this.remove();
			throw new PublishException("cannot_publish_two_remoteresources_with_same_shape");
		}			
	}

	/**
	 * TODO: at the moment only one namespace is possible (default)
	 * @return
	 */
	private GSNamespace create_namespace(){
		GSNamespace ns = new GSNamespace();		
		ns.setPrefix(getNamespacePrefix());
		try {
			ns.setURI(new URI(getService().getServer().getServerURL().toString()));
		} catch (URISyntaxException e1) {			
			PublishLogger.getLog().error("ERROR " + getClassName() + ":uri syntax error", e1);
		}
		return ns;
	}
	/**
	 * 
	 * @param ns
	 * @return
	 */
	private GSDatastore create_datastore(GSFeature feature, GSNamespace ns) {
		GSDatastore ds = new GSDatastore();
		ds.setNamespace(ns);
		IDataSourceInfo dataSrc = getLayerInfo().getDataSource();		
		if (dataSrc.getType().equals(IDataSourceInfo.SHAPE_TYPE)){			
			config_datastore_shape(feature, ds);
		}
		if (dataSrc.getType().equals(IDataSourceInfo.POSTGIS_TYPE)){			
			config_datastore_postgis(feature, ds);
		}
		return ds;
	}
	/**
	 * Creates the datastore configuration for Geoserver
	 * 
	 * @param feature
	 * @param ds
	 */
	private void config_datastore_postgis(GSFeature feature, GSDatastore ds) {
		String ds_id  = ((IDataBaseInfo)getLayerInfo().getDataSource()).getConnectionName();
		ds_id  = ds_id + "_" + ((IDataBaseInfo)getLayerInfo().getDataSource()).getSchemaName();
		ds.setId(ds_id);
		ds.setAbstract("Datastore description generated by gvSIG");
		ds.setLooseBbox(true);
		ds.setUser(((IDataBaseInfo)getLayerInfo().getDataSource()).getUser());
		ds.setPassword(((IDataBaseInfo)getLayerInfo().getDataSource()).getPassword());
		ds.setEstimatedExtent(false);
		ds.setHost(((IDataBaseInfo)getLayerInfo().getDataSource()).getHost());
		ds.setSchema(((IDataBaseInfo)getLayerInfo().getDataSource()).getSchemaName());
		ds.setPort(new Integer(((IDataBaseInfo)getLayerInfo().getDataSource()).getPort()).intValue());
		ds.setDatabase(((IDataBaseInfo)getLayerInfo().getDataSource()).getDatabaseName());
		ds.setDbType("postgis");		
		//the feature id is namespace prefix + "_" + tablename
		String id = getLayerInfo().getDataSource().getNamespacePrefix() + "_" + ((IDataBaseInfo)getLayerInfo().getDataSource()).getTableName();
		feature.setId(id);
	}
	/**
	 * Creates the datastore configuration for Geoserver
	 * @param feature
	 * @param ds
	 * 
	 */
	private void config_datastore_shape(GSFeature feature, GSDatastore ds) {

		ds.setId(((IFileInfo)getLayerInfo().getDataSource()).getFilenameWithoutExtension());
		//TODO: get this from IDatasourceInfo
		ds.setCharset("UTF-8");		
		try {
			IDataSourceInfo datasource = getLayerInfo().getDataSource();
			String f = getService().getServer().getPublication().getServerSideFilePath(datasource);
			ds.setURL(new URL("file:" + f));
		} catch (MalformedURLException e) {
			PublishLogger.getLog().error("ERROR: " + getClassName() + ":malformed url", e);
		}	
		//the feature id is namespace + "_" + filename without extension
		String id = getLayerInfo().getDataSource().getNamespacePrefix() + "_" + getId(); 
		feature.setId(id);
	}
	/**
	 * Creates the style configuration for Geoserver 
	 * 
	 * @return
	 * 
	 */
	private GSStyle create_style(){
		//create style
		GSStyle st = new GSStyle();
		st.setId(getId());
		if (getLayerInfo().getLegend()!= null){
			st.setSLDString(getLayerInfo().getLegend().getSLDString());
		}
		return st;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.RemoteResource#getRegisterTag()
	 */
	public String getRegisterTag() {

		return REGISTER_TAG;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getAbstract()
	 */
	public String getAbstract() {

		return ogcMetadata.getAbstract();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getName()
	 */
	public String getName() {

		return ogcMetadata.getName();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getTitle()
	 */
	public String getTitle() {

		return ogcMetadata.getTitle();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setAbstract(java.lang.String)
	 */
	public void setAbstract(String description) {
		ogcMetadata.setAbstract(description);		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setName(java.lang.String)
	 */
	public void setName(String name) {
		ogcMetadata.setName(name);

	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setTitle(java.lang.String)
	 */
	public void setTitle(String title) {
		ogcMetadata.setTitle(title);		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion() {		
		return 1;
	}
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return getTitle();
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.RemoteResource#getXMLEntity()
	 */

	public XMLEntity getXMLEntity() {
		XMLEntity xml=super.getXMLEntity();
		//put version and name
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		//put associations
		XMLEntity child = new XMLEntity();
		child.setName("ogcmetadata");
		child.addChild(ogcMetadata.getXMLEntity());
		xml.addChild(child);
		//put config properties
		xml.putProperty("namespace_prefix", getNamespacePrefix());
		xml.putProperty("is_cached", isCacheEnabled());
		xml.putProperty("cache_maxage", getCacheMaxage());
		return xml;
	}

	/* (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.RemoteResource#setXMLEntity(com.iver.utiles.XMLEntity)
	 */

	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");
		if (version != getVersion()){
			PublishLogger.getLog().error("ERROR: " + getClassName() + ": the version doesn't match!");			
			return;
		}
		super.setXMLEntity(xml);						
		//set associations
		XMLEntity child = xml.firstChild("name","ogcmetadata");
		ogcMetadata.setXMLEntity(child.getChild(0));
		//set config properties
		setNamespacePrefix(xml.getStringProperty("namespace_prefix"));
		setCacheEnabled(xml.getBooleanProperty("is_cached"));
		setCacheMaxage(xml.getIntProperty("cache_maxage"));
	}

	/**
	 * @return the cacheEnabled
	 */
	public boolean isCacheEnabled() {
		return cacheEnabled;
	}

	/**
	 * @param cacheEnabled the cacheEnabled to set
	 */
	public void setCacheEnabled(boolean cacheEnabled) {
		this.cacheEnabled = cacheEnabled;
	}

	/**
	 * @return the cacheMaxage
	 */
	public int getCacheMaxage() {
		return cacheMaxage;
	}

	/**
	 * @param cacheMaxage the cacheMaxage to set
	 */
	public void setCacheMaxage(int cacheMaxage) {
		this.cacheMaxage = cacheMaxage;
	}

	/**
	 * @return the numDecimals
	 */
	public int getNumDecimals() {
		return numDecimals;
	}

	/**
	 * @param numDecimals the numDecimals to set
	 */
	public void setNumDecimals(int numDecimals) {
		this.numDecimals = numDecimals;
	}

	/**
	 * @return the namespacePrefix
	 */
	public String getNamespacePrefix() {
		return namespacePrefix;
	}

	/**
	 * @param namespacePrefix the namespacePrefix to set
	 */
	public void setNamespacePrefix(String namespacePrefix) {
		this.namespacePrefix = namespacePrefix;
	}

}
