/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibañez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.publish.geoserver.model;

import java.net.MalformedURLException;
import java.net.URL;

import org.gvsig.publish.PublishLogger;
import org.gvsig.publish.exceptions.PublishException;
import org.gvsig.publish.infoproject.FilterProjectInfo;
import org.gvsig.publish.infoproject.IDataSourceInfo;
import org.gvsig.publish.ogcmetadata.IOGCServiceMetadata;
import org.gvsig.publish.ogcmetadata.OGCServiceMetadata;
import org.gvsig.publish.serversmodel.Service;

import com.iver.utiles.XMLEntity;
/**
 * This class represents the Geoserver WFS Service.
 * 
 * Responsibilities:
 *  Persist its attributes.
 *  Generate the configuration related to the Geoserver WMS service.
 * NOTE:
 *  For the correct working of the server you must generate a WMS configuration
 *  so this class has this association.
 *  The service WMS is added to the server in the first publication because the
 *  server use a LinkedHashMap to store its services and WFS must be the first 
 *  (the GUI is developed assuming that a server has only one service, the first 
 *  in the list). 
 * 
 * @author jvhigon (josevicente.higon@iver.es)
 *
 */
public class GeoserverWFSService extends Service implements IOGCServiceMetadata {
	//constants 
	public static final String REGISTER_TAG="WFS 1.0.0";
	public static final int LEVEL_BASIC=1;
	public static final int LEVEL_TRANSACTIONAL=15;
	public static final int LEVEL_COMPLETE=31;
	//common attributes
	private OGCServiceMetadata ogcMetadata= null;
	private FilterProjectInfo filter = null;
	//Geoserver attributes
	private boolean enabled=false;
	private int serviceLevel=LEVEL_COMPLETE;
	//associations
	private GeoserverWMSService geoserverwms=null;


	/**
	 * Constructor
	 * 
	 * @param server
	 */
	public GeoserverWFSService(Geoserver server) {
		super(server);				
		ogcMetadata = new OGCServiceMetadata();		
		initialize();				
	}
	/**
	 * Initialize some variables with the server information. For example, URLs.
	 */
	private void initialize(){
		//setting url
		URL url = getServer().getServerURL();
		URL aux_wfs=null;	
		try {
			aux_wfs = new URL(url.toString() + "/wfs");
		} catch (MalformedURLException e) {
			PublishLogger.getLog().error("what are you doing my friend!", e);
		}catch (NullPointerException e){
			PublishLogger.getLog().warn("WARNING " + getClassName() +  ":The server don't have URL",e );
		}
			
		//setting name and title
		setName("OGC:WFS");
		setTitle("Geoserver WFS 1.0.0");
		setAbstract("generated by gvSIG");
		setOnlineResource(aux_wfs);		
	}
	/**
	 * @return string which identifies the class and its version
	 */
	public String getClassName() {
		return "GeoserverWFSService";
	}
	
	/**
	 * I add the wms service in the first publication!!
	 * 
	 * @see org.gvsig.publish.serversmodel.Service#publish()
	 */
	public void publish() throws PublishException {
		Geoserver server = (Geoserver)getServer();
		//TODO: I must quit this dependence
		if (geoserverwms == null){
			geoserverwms = new GeoserverWMSService(server);
			//server.addService(geoserverwms);			
		}	
		geoserverwms.publish();
		//generate services configuration
		server.getConfiguration().getGlobals().setWFSService(this);		
		//generate remote resources configuration
		super.publish();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.Service#getFilter()
	 */
	public FilterProjectInfo getFilter() {
		if (filter == null){
			filter = new FilterProjectInfo();
		}
		filter.setGroupedLayersEnabled(false);
		filter.addAllowedDatasource(IDataSourceInfo.POSTGIS_TYPE);
		filter.addAllowedDatasource(IDataSourceInfo.SHAPE_TYPE);		
		return this.filter;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.Service#getId()
	 */
	public String getId() {
		
		return "geoserver_wfs";
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.Service#getRegisterTag()
	 */
	public String getRegisterTag() {		
		return REGISTER_TAG;
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCServiceMetadata#getOnlineResource()
	 */
	public URL getOnlineResource() {
		
		return ogcMetadata.getOnlineResource();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCServiceMetadata#setOnlineResource(java.net.URL)
	 */
	public void setOnlineResource(URL url) {
		ogcMetadata.setOnlineResource(url);
		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getAbstract()
	 */
	public String getAbstract() {
		
		return ogcMetadata.getAbstract();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getName()
	 */
	public String getName() {
		
		return ogcMetadata.getName();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#getTitle()
	 */
	public String getTitle() {
		
		return ogcMetadata.getTitle();
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setAbstract(java.lang.String)
	 */
	public void setAbstract(String description) {
		ogcMetadata.setAbstract(description);
		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setName(java.lang.String)
	 */
	public void setName(String name) {
		ogcMetadata.setName(name);
		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.ogcmetadata.IOGCCommonMetadata#setTitle(java.lang.String)
	 */
	public void setTitle(String title) {
		ogcMetadata.setTitle(title);
		
	}
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.publish.IPublishPersistence#getVersion()
	 */
	public int getVersion() {
		
		return 1;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.Service#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml=super.getXMLEntity();
		//put version and name 
		xml.setName(getClassName());
		xml.putProperty("version", getVersion());
		//pùt its associations
		XMLEntity child = new XMLEntity();
		child.setName("wfsmetadata");
		child.addChild(ogcMetadata.getXMLEntity());		
		xml.addChild(child);
		//put config properties
		xml.putProperty("service_level", getServiceLevel());
		return xml;
	}
	/* (non-Javadoc)
	 * @see org.gvsig.publish.serversmodel.Service#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml) {
		//check version
		int version = xml.getIntProperty("version");
		if (version != getVersion()){
			PublishLogger.getLog().error("ERROR: " + getClassName()+ ": the version doesn't match!");
			return;
		}	
		super.setXMLEntity(xml);
		
		//set associations
		XMLEntity child = xml.firstChild("name","wfsmetadata");
		ogcMetadata.setXMLEntity(child.getChild(0));
		//set config properties
		setServiceLevel(xml.getIntProperty("service_level"));
	}
	/**
	 * @return the enabled
	 */
	public boolean isEnabled() {
		return enabled;
	}
	/**
	 * @param enabled the enabled to set
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	/**
	 * @return the serviceLevel
	 */
	public int getServiceLevel() {
		return serviceLevel;
	}
	/**
	 * @param serviceLevel the serviceLevel to set
	 */
	public void setServiceLevel(int serviceLevel) {
		this.serviceLevel = serviceLevel;
	}
		
}
